/**
Generate your own custom-cookie-cutters<br/>
by <a href="http://www.local-guru.net/blog">Guru</a>
*/

import processing.opengl.*;
import unlekker.data.*;
PVector[] p;

void setup() {
  size(300,300,OPENGL);
  smooth();
  
  p = new PVector[20];
  
  for( int i=0; i < p.length; i++) {
    float r = 30 + random(50);
    p[i] = new PVector( 150  + cos( radians( i * 18 )) * r, 150 + sin( radians( i * 18 )) * r );
  }
}

int dragged = -1;

boolean ex = false;

void draw() {
  if (!ex) {
    edit();  
  } else {
    export();
  } 
}

void keyPressed() {
  if ( key == 'e' ) {
    noLoop();
    export();
    loop();
  }
}

void edit() {
  background(128);
  fill(255);
  noStroke();

  beginShape( TRIANGLE_STRIP );  
  for( int i = 0; i<p.length + 1; i++ ) {
    int last = (i - 1) % p.length;
    if (last < 0) last = p.length - 1;
    int next = (i + 1) % p.length;
    
    PVector tmp = new PVector( p[last].x, p[last].y );
    tmp.sub( p[next] );
    PVector n = new PVector( tmp.y, -tmp.x );
    
    vertex( p[i % p.length].x + n.x * 20 / p[last].dist(p[next]) , p[i % p.length].y + n.y * 20 / p[last].dist(p[next]) );
    vertex( p[i % p.length].x - n.x * 20 / p[last].dist(p[next]) , p[i % p.length].y - n.y * 20 / p[last].dist(p[next]) );
  }
  endShape(CLOSE);

  noFill();
  stroke(1); 
  
  beginShape();  
  for( int i = 0; i<p.length; i++ ) {
    if (mousePressed && mouseButton == LEFT && dragged < 0 ) {
      if (p[i].dist(new PVector(mouseX, mouseY)) < 4) {
        dragged = i;
      }
    } else if ( mousePressed && mouseButton == LEFT && dragged == i ) {
      p[i].x = mouseX;
      p[i].y = mouseY;
    }
    if ( !mousePressed ) {
      dragged =-1;
    }
    vertex( p[i].x, p[i].y );
  }
  endShape(CLOSE);
  fill(0);
  for( int i = 0; i<p.length; i++ ) {
    ellipse( p[i].x, p[i].y, 4, 4);
  }  
}

void export() {
  float r1 = 10;
  float r2 = 1;
  float h1 = 4;
  float h2 = 40;
  float sc = 0.25;

  println( "EXPORT");
  beginRaw("unlekker.data.STL","cookiecutter.stl");
  translate( 115,115,260);
  scale( sc );
  //top  
  beginShape( TRIANGLE_STRIP );  
  for( int i = 0; i<p.length + 1; i++ ) {
    int last = (i - 1) % p.length;
    if (last < 0) last = p.length - 1;
    int next = (i + 1) % p.length;
    
    PVector tmp = new PVector( p[last].x, p[last].y );
    tmp.sub( p[next] );
    PVector n = new PVector( tmp.y, -tmp.x );
    
    vertex( p[i % p.length].x + n.x * r1 / p[last].dist(p[next]), p[i % p.length].y + n.y * r1 / p[last].dist(p[next]),0);
    vertex( p[i % p.length].x - n.x * r1 / p[last].dist(p[next]), p[i % p.length].y - n.y * r1 / p[last].dist(p[next]),0);
  }
  endShape(CLOSE);
  //inner wall
  beginShape( QUAD_STRIP );  
  for( int i = 0; i<p.length + 1; i++ ) {
    int last = (i - 1) % p.length;
    if (last < 0) last = p.length - 1;
    int next = (i + 1) % p.length;
    
    PVector tmp = new PVector( p[last].x, p[last].y );
    tmp.sub( p[next] );
    PVector n = new PVector( tmp.y, -tmp.x );
    
    vertex( p[i % p.length].x + n.x * r1 / p[last].dist(p[next]) , p[i % p.length].y + n.y * r1 / p[last].dist(p[next]),0);
    vertex( p[i % p.length].x + n.x * r1 / p[last].dist(p[next]) , p[i % p.length].y + n.y * r1 / p[last].dist(p[next]),h1);

  }
  endShape(CLOSE);
  //outer wall
  beginShape( QUAD_STRIP );  
  for( int i = 0; i<p.length + 1; i++ ) {
    int last = (i - 1) % p.length;
    if (last < 0) last = p.length - 1;
    int next = (i + 1) % p.length;
    
    PVector tmp = new PVector( p[last].x, p[last].y );
    tmp.sub( p[next] );
    PVector n = new PVector( tmp.y, -tmp.x );
    
    vertex( p[i % p.length].x - n.x * r1 / p[last].dist(p[next]) , p[i % p.length].y - n.y * r1 / p[last].dist(p[next]),0);
    vertex( p[i % p.length].x - n.x * r1 / p[last].dist(p[next]) , p[i % p.length].y - n.y * r1 / p[last].dist(p[next]),h1);
  }
  endShape(CLOSE);
  
  //top  
  beginShape( TRIANGLE_STRIP );  
  for( int i = 0; i<p.length + 1; i++ ) {
    int last = (i - 1) % p.length;
    if (last < 0) last = p.length - 1;
    int next = (i + 1) % p.length;
    
    PVector tmp = new PVector( p[last].x, p[last].y );
    tmp.sub( p[next] );
    PVector n = new PVector( tmp.y, -tmp.x );
    
    vertex( p[i % p.length].x + n.x * r1 / p[last].dist(p[next]) , p[i % p.length].y + n.y * r1 / p[last].dist(p[next]),h1);
    vertex( p[i % p.length].x - n.x * r1 / p[last].dist(p[next]) , p[i % p.length].y - n.y * r1 / p[last].dist(p[next]),h1);
  }
  endShape(CLOSE);

   //cutting part inner wall
  beginShape( QUAD_STRIP );  
  for( int i = 0; i<p.length + 1; i++ ) {
    int last = (i - 1) % p.length;
    if (last < 0) last = p.length - 1;
    int next = (i + 1) % p.length;
    
    PVector tmp = new PVector( p[last].x, p[last].y );
    tmp.sub( p[next] );
    PVector n = new PVector( tmp.y, -tmp.x );
    
    vertex( p[i % p.length].x + n.x * r2 / p[last].dist(p[next]) , p[i % p.length].y + n.y * r2 / p[last].dist(p[next]),h1);
    vertex( p[i % p.length].x + n.x * r2 / p[last].dist(p[next]) , p[i % p.length].y + n.y * r2 / p[last].dist(p[next]),h2);

  }
  endShape(CLOSE);
  //cutting part outer wall
  beginShape( QUAD_STRIP );  
  for( int i = 0; i<p.length + 1; i++ ) {
    int last = (i - 1) % p.length;
    if (last < 0) last = p.length - 1;
    int next = (i + 1) % p.length;
    
    PVector tmp = new PVector( p[last].x, p[last].y );
    tmp.sub( p[next] );
    PVector n = new PVector( tmp.y, -tmp.x );
    
    vertex( p[i % p.length].x - n.x * r2 / p[last].dist(p[next]) , p[i % p.length].y - n.y * r2 / p[last].dist(p[next]),h1);
    vertex( p[i % p.length].x - n.x * r2 / p[last].dist(p[next]) , p[i % p.length].y - n.y * r2 / p[last].dist(p[next]),h2);
  }
  endShape(CLOSE);
 
  //cutting part top  
  beginShape( TRIANGLE_STRIP );  
  for( int i = 0; i<p.length + 1; i++ ) {
    int last = (i - 1) % p.length;
    if (last < 0) last = p.length - 1;
    int next = (i + 1) % p.length;
    
    PVector tmp = new PVector( p[last].x, p[last].y );
    tmp.sub( p[next] );
    PVector n = new PVector( tmp.y, -tmp.x );
    
    vertex( p[i % p.length].x + n.x * r2 / p[last].dist(p[next]) , p[i % p.length].y + n.y * r2 / p[last].dist(p[next]),h2);
    vertex( p[i % p.length].x - n.x * r2 / p[last].dist(p[next]) , p[i % p.length].y - n.y * r2 / p[last].dist(p[next]),h2);
  }
  endShape(CLOSE);
 
  
  endRaw();
}
